
#include <ruby.h>
#include <liblwgeom.h>

#define FFI_LWGEOM_GEOS_ERRMSG_MAXSIZE 256
char lwgeom_geos_errmsg[FFI_LWGEOM_GEOS_ERRMSG_MAXSIZE];

static ID id_LWGeom;
static ID id_LWGeom_Error;

static void ffi_lwgeom_notice(const char *fmt, va_list ap) {
  char errmsg[FFI_LWGEOM_GEOS_ERRMSG_MAXSIZE + 1];

  if (rb_gv_get(("$DEBUG"))) {
    vsnprintf(errmsg, FFI_LWGEOM_GEOS_ERRMSG_MAXSIZE, fmt, ap);
    fprintf(stderr, "lwgeom internal notice: %s\n", errmsg);
  }
}

static void ffi_lwgeom_error(const char *fmt, va_list ap) {
  VALUE lwgeom = rb_const_get(rb_cObject, id_LWGeom);
  VALUE lwgeom_error = rb_const_get(lwgeom, id_LWGeom_Error);
  char errmsg[FFI_LWGEOM_GEOS_ERRMSG_MAXSIZE + 1];

  vsnprintf(errmsg, FFI_LWGEOM_GEOS_ERRMSG_MAXSIZE, fmt, ap);
  rb_raise(lwgeom_error, "lwgeom internal error: %s", errmsg);
}

void lwgeom_init_allocators(void) {
  lwalloc_var = default_allocator;
  lwrealloc_var = default_reallocator;
  lwfree_var = default_freeor;
  lwnotice_var = ffi_lwgeom_notice;
  lwerror_var = ffi_lwgeom_error;
}

void Init_ffi_lwgeom_impl() {
  id_LWGeom = rb_intern("LWGeom");
  id_LWGeom_Error = rb_intern("Error");
}

